-----------------------------------------------------------------------------------------------
-- Client Lua Script for SlimUnitFrame
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
-- Special thanks to Sinaloit and Jos_eu / Rimoko for the help

require "Window"
require "Unit"
require "GameLib"
require "Apollo"

-----------------------------------------------------------------------------------------------
-- SlimUnitFrame Module Definition
-----------------------------------------------------------------------------------------------

local SlimUnitFrame = {} 
local unitFramesAddon = Apollo.GetAddon("TargetFrame")
local runOnce = true
local ok = 0
local unitFramesAddon_OnDocumentReady = nil

local commands = {
					"bg show",
					"bg hide",
					"bg h",
					"bg s",
					"simplehp on",
					"simplehp off",
					"show",
					"hide",
					"s",
					"h",
					"showlevel on",
					"showlevel off",
					"shp on",
					"shp off",
					"slvl on",
					"slvl off",
					"percentage on",
					"% on",
					"percentage off",
					"% off",
					"showclass on",
					"showclass off",
					"animatedmodel on",
					"animatedmodel off",
}	

-- User settings
local simpleHP 
local showPortraits 
local showBackground
local showLevel
local showTotalPercentage
local alternativeFont
local showClass
local animatedModel
local fadeOutCombat

 ---- Duplicated TargetFrame variables
 
local knMaxLevel							= 50
local knFrameWidthMax	 					= 400
local knFrameWidthShield	 				= 372
local knFrameWidthMin	 					= 340
local knClusterFrameWidth 					= 60 -- MUST MATCH XML
local knClusterFrameHeight 					= 62 -- MUST MATCH XML
local knClusterFrameVertOffset 				= 120 -- how far down to move the cluster members
local knHealthRed							= 0.3
local knHealthYellow						= 0.5

local kstrScalingHex = "ffffbf80"
local kcrScalingCColor = CColor.new(1.0, 191/255, 128/255, 0.7)

local karDispositionColors =
{
	[Unit.CodeEnumDisposition.Neutral]  = ApolloColor.new("DispositionNeutral"),
	[Unit.CodeEnumDisposition.Hostile]  = ApolloColor.new("FFFF5757"), -- New color for hostiles NPCs
	[Unit.CodeEnumDisposition.Friendly] = ApolloColor.new("DispositionFriendly"),
}

local ktDispositionToTTooltip =
{
	[Unit.CodeEnumDisposition.Neutral]  = Apollo.GetString("TargetFrame_NeutralTooltip"),
	[Unit.CodeEnumDisposition.Hostile]  = Apollo.GetString("TargetFrame_HostileTooltip"),
	[Unit.CodeEnumDisposition.Friendly] = Apollo.GetString("TargetFrame_FriendlyTooltip"),
}

local kstrRaidMarkerToSprite =
{
	"Icon_Windows_UI_CRB_Marker_Bomb",
	"Icon_Windows_UI_CRB_Marker_Ghost",
	"Icon_Windows_UI_CRB_Marker_Mask",
	"Icon_Windows_UI_CRB_Marker_Octopus",
	"Icon_Windows_UI_CRB_Marker_Pig",
	"Icon_Windows_UI_CRB_Marker_Chicken",
	"Icon_Windows_UI_CRB_Marker_Toaster",
	"Icon_Windows_UI_CRB_Marker_UFO",
}

local karFactionToString = --Used for the Attachment Frame Sprites
{
	[Unit.CodeEnumFaction.ExilesPlayer]		= "Exile",
	[171]									= "Exile", --Exile NPC's
	
	[Unit.CodeEnumFaction.DominionPlayer]	= "Dominion",
	[170]									= "Dominion", --Dominion NPC's
}

-- Todo: break these out onto options
local kcrGroupTextColor					= ApolloColor.new("crayBlizzardBlue")
local kcrFlaggedFriendlyTextColor 		= karDispositionColors[Unit.CodeEnumDisposition.Friendly]
local kcrDefaultGuildmemberTextColor 	= karDispositionColors[Unit.CodeEnumDisposition.Friendly]
local kcrHostileEnemyTextColor 			= karDispositionColors[Unit.CodeEnumDisposition.Hostile]
local kcrAggressiveEnemyTextColor 		= karDispositionColors[Unit.CodeEnumDisposition.Neutral]
local kcrNeutralEnemyTextColor 			= ApolloColor.new("crayDenim")
local kcrDefaultUnflaggedAllyTextColor 	= karDispositionColors[Unit.CodeEnumDisposition.Friendly]

-- TODO:Localize all of these
-- differential value, color, title, description, title color (for tooltip)
local karConInfo =
{
	{-4, ApolloColor.new("ConTrivial"), 	Apollo.GetString("TargetFrame_Trivial"), 	Apollo.GetString("TargetFrame_NoXP"), 				"ff7d7d7d"},
	{-3, ApolloColor.new("ConInferior"), 	Apollo.GetString("TargetFrame_Inferior"), 	Apollo.GetString("TargetFrame_VeryReducedXP"), 		"ff01ff07"},
	{-2, ApolloColor.new("ConMinor"), 		Apollo.GetString("TargetFrame_Minor"), 		Apollo.GetString("TargetFrame_ReducedXP"), 			"ff01fcff"},
	{-1, ApolloColor.new("ConEasy"), 		Apollo.GetString("TargetFrame_Easy"), 		Apollo.GetString("TargetFrame_SlightlyReducedXP"), 	"ff597cff"},
	{ 0, ApolloColor.new("ConAverage"), 	Apollo.GetString("TargetFrame_Average"), 	Apollo.GetString("TargetFrame_StandardXP"), 		"ffffffff"},
	{ 1, ApolloColor.new("ConModerate"), 	Apollo.GetString("TargetFrame_Moderate"), 	Apollo.GetString("TargetFrame_SlightlyMoreXP"), 	"ffffff00"},
	{ 2, ApolloColor.new("ConTough"), 		Apollo.GetString("TargetFrame_Tough"), 		Apollo.GetString("TargetFrame_IncreasedXP"), 		"ffff8000"},
	{ 3, ApolloColor.new("ConHard"), 		Apollo.GetString("TargetFrame_Hard"), 		Apollo.GetString("TargetFrame_HighlyIncreasedXP"), 	"ffff0000"},
	{ 4, ApolloColor.new("ConImpossible"), 	Apollo.GetString("TargetFrame_Impossible"), Apollo.GetString("TargetFrame_GreatlyIncreasedXP"),	"ffff00ff"}
}

-- Todo: Localize
local ktRankDescriptions =
{
	[Unit.CodeEnumRank.Fodder] 		= 	{Apollo.GetString("TargetFrame_Fodder"), 		Apollo.GetString("TargetFrame_VeryWeak")},
	[Unit.CodeEnumRank.Minion] 		= 	{Apollo.GetString("TargetFrame_Minion"), 		Apollo.GetString("TargetFrame_Weak")},
	[Unit.CodeEnumRank.Standard]	= 	{Apollo.GetString("TargetFrame_Grunt"), 		Apollo.GetString("TargetFrame_EasyAppend")},
	[Unit.CodeEnumRank.Champion] 	=	{Apollo.GetString("TargetFrame_Challenger"), 	Apollo.GetString("TargetFrame_AlmostEqual")},
	[Unit.CodeEnumRank.Superior] 	=  	{Apollo.GetString("TargetFrame_Superior"), 		Apollo.GetString("TargetFrame_Strong")},
	[Unit.CodeEnumRank.Elite] 		= 	{Apollo.GetString("TargetFrame_Prime"), 		Apollo.GetString("TargetFrame_VeryStrong")},
}

local karClassToIcon =
{
	[GameLib.CodeEnumClass.Warrior] 		= "IconSprites:Icon_Windows_UI_CRB_Warrior",
	[GameLib.CodeEnumClass.Engineer] 		= "IconSprites:Icon_Windows_UI_CRB_Engineer",
	[GameLib.CodeEnumClass.Esper] 			= "IconSprites:Icon_Windows_UI_CRB_Esper",
	[GameLib.CodeEnumClass.Medic] 			= "IconSprites:Icon_Windows_UI_CRB_Medic",
	[GameLib.CodeEnumClass.Stalker] 		= "IconSprites:Icon_Windows_UI_CRB_Stalker",
	[GameLib.CodeEnumClass.Spellslinger] 	= "IconSprites:Icon_Windows_UI_CRB_Spellslinger",
}

local kstrTooltipBodyColor = "ffc0c0c0"
local kstrTooltipTitleColor = "ffdadada"

local kstrFriendSprite 			= "ClientSprites:Icon_Windows_UI_CRB_Friend"
local kstrAccountFriendSprite 	= "ClientSprites:Icon_Windows_UI_CRB_Friend"
local kstrRivalSprite 			= "ClientSprites:Icon_Windows_UI_CRB_Rival"
 
 -----
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SlimUnitFrame:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    return o
end

function SlimUnitFrame:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "Slim UnitFrame"
	local tDependencies = {"TargetFrame"}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
	self.LoadDefaultSettings()
end
 
-----------------------------------------------------------------------------------------------
-- SlimUnitFrame OnLoad
-----------------------------------------------------------------------------------------------
function SlimUnitFrame:OnLoad()

	self.onRestoreCalled = false
	UnitFramesBase_OnDocumentReady = unitFramesAddon.OnDocumentReady
	unitFramesAddon.OnDocumentReady = SlimUnitFrame.UnitFramesHook_OnDocumentReady
	
	Apollo.RegisterTimerHandler("OneSecTimer", "OnTimer", self)
	Apollo.RegisterSlashCommand("suf", "onSlashCommand", self)
	Apollo.RegisterSlashCommand("slimunitframe", "onSlashCommand", self)
	
	 -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("SlimUnitFrame.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
		
end

function SlimUnitFrame:LoadDefaultSettings()
	simpleHP = true
	showPortraits = true
	showBackground = true
	showLevel = true
	showTotalPercentage = true
	alternativeFont = true
	showClass = true
	animatedModel = false
	fadeOutCombat = false
end
 
function SlimUnitFrame.UnitFramesHook_OnDocumentReady(unitFramesSelf)
	UnitFramesBase_OnDocumentReady(unitFramesSelf)  -- this will create the luaTargetFrame
	local unitFrames = unitFramesSelf.luaUnitFrame
	unitFrames.ArrangeClusterMembers = SlimUnitFrame.ArrangeClusterMembers
	unitFrames.UpdateCastingBar = SlimUnitFrame.UpdateCastingBar
	unitFrames.SetTargetHealthAndShields = SlimUnitFrame.SetTargetHealthAndShields
	unitFrames.SetTargetForFrame = SlimUnitFrame.SetTargetForFrame
	unitFrames = unitFramesSelf.luaTargetFrame
	unitFrames.ArrangeClusterMembers = SlimUnitFrame.ArrangeClusterMembers
	unitFrames.UpdateCastingBar = SlimUnitFrame.UpdateCastingBar
	unitFrames.SetTargetHealthAndShields = SlimUnitFrame.SetTargetHealthAndShields
	unitFrames.SetTargetForFrame = SlimUnitFrame.SetTargetForFrame
	unitFrames = unitFramesSelf.luaFocusFrame
	unitFrames.ArrangeClusterMembers = SlimUnitFrame.ArrangeClusterMembers
	unitFrames.UpdateCastingBar = SlimUnitFrame.UpdateCastingBar
	unitFrames.SetTargetHealthAndShields = SlimUnitFrame.SetTargetHealthAndShields
	unitFrames.SetTargetForFrame = SlimUnitFrame.SetTargetForFrame		
end

-----------------------------------------------------------------------------------------------
-- SlimUnitFrame OnDocLoaded
-----------------------------------------------------------------------------------------------
function SlimUnitFrame:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndSettings = Apollo.LoadForm(self.xmlDoc, "SUFSettings", nil, self)
		if self.wndSettings == nil then
			Apollo.AddAddonErrorText(self, "Could not load the settings window for some reason.")
			return
		end	
		
	    self.wndSettings:Show(false, true)
		
		self.onXmlDocLoadedCalled = true
		self:UpdateSettings()
	end
end

-----------------------------------------------------------------------------------------------
-- SlimUnitFrame Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

function SlimUnitFrame:onSlashCommand(slashcommand, subcommand)
	-- Show Portraits
	if subcommand == "show" or subcommand == "s" or showPortraits == true then
		if unitFramesAddon then
			if unitFramesAddon.luaUnitFrame then
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("Window"):SetOpacity(1)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetModel"):SetOpacity(1)
			end
			if unitFramesAddon.luaTargetFrame then
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("Window"):SetOpacity(1)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetModel"):SetOpacity(1)
				unitFramesAddon.luaTargetFrame["wndToTFrame"]:FindChild("TargetModel"):Show(true)
				unitFramesAddon.luaTargetFrame["wndToTFrame"]:FindChild("Backer"):Show(true)
			end
			if unitFramesAddon.luaFocusFrame then
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("Window"):SetOpacity(1)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetModel"):SetOpacity(1)
				unitFramesAddon.luaFocusFrame["wndToTFrame"]:FindChild("TargetModel"):Show(true)
				unitFramesAddon.luaFocusFrame["wndToTFrame"]:FindChild("Backer"):Show(true)
			end
		end	
		showPortraits = true
	end
	-- Hide Portraits
	if subcommand == "hide" or subcommand == "h" or showPortraits == false then	
		if unitFramesAddon then
			if unitFramesAddon.luaUnitFrame then
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("Window"):Show(false)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetModel"):Show(false)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("Window"):SetOpacity(0)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetModel"):SetOpacity(0)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("Window"):Show(true)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetModel"):Show(true)
			end
			if unitFramesAddon.luaTargetFrame then
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("Window"):Show(false)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetModel"):Show(false)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("Window"):SetOpacity(0)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetModel"):SetOpacity(0)
				unitFramesAddon.luaTargetFrame["wndToTFrame"]:FindChild("TargetModel"):Show(false)
				unitFramesAddon.luaTargetFrame["wndToTFrame"]:FindChild("Backer"):Show(false)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("Window"):Show(true)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetModel"):Show(true)
			end
			if unitFramesAddon.luaFocusFrame then
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("Window"):Show(false)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetModel"):Show(false)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("Window"):SetOpacity(0)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetModel"):SetOpacity(0)
				unitFramesAddon.luaFocusFrame["wndToTFrame"]:FindChild("TargetModel"):Show(false)
				unitFramesAddon.luaFocusFrame["wndToTFrame"]:FindChild("Backer"):Show(false)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("Window"):Show(true)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetModel"):Show(true)
			end
		end	
		showPortraits = false
	end	
	
	-- Show background
	if subcommand == "bg show" or subcommand == "bg s" or showBackground == true then
		if unitFramesAddon then
			if unitFramesAddon.luaUnitFrame then
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("CastingBase"):SetSprite("BK3:sprGradientOutwards")
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("CastingBase"):SetBGColor("UI_AlphaPercent25")
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetSprite("BK3:sprGradientOutwards")
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetBGColor("UI_AlphaPercent25")
			end
			if unitFramesAddon.luaTargetFrame then
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("CastingBase"):SetSprite("BK3:sprGradientOutwards")
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("CastingBase"):SetBGColor("UI_AlphaPercent25")
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetSprite("BK3:sprGradientOutwards")
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetBGColor("UI_AlphaPercent25")
				unitFramesAddon.luaTargetFrame["wndToTFrame"]:FindChild("Backer"):SetSprite("sprGradientOutwards")
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("Backer"):SetBGColor("UI_AlphaPercent25")
			end
			if unitFramesAddon.luaFocusFrame then
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("CastingBase"):SetSprite("BK3:sprGradientOutwards")
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("CastingBase"):SetBGColor("UI_AlphaPercent25")
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetSprite("BK3:sprGradientOutwards")
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetBGColor("UI_AlphaPercent25")
				unitFramesAddon.luaFocusFrame["wndToTFrame"]:FindChild("Backer"):SetSprite("sprGradientOutwards")
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("Backer"):SetBGColor("UI_AlphaPercent25")
			end
		end	
		showBackground = true
	end
	
	-- Hide background
	if subcommand == "bg hide" or subcommand == "bg h" or showBackground == false then
		if unitFramesAddon then
			if unitFramesAddon.luaUnitFrame then
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetSprite("")
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("CastingBase"):SetSprite("")
			end
			if unitFramesAddon.luaTargetFrame then
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetSprite("")
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("CastingBase"):SetSprite("")
				unitFramesAddon.luaTargetFrame["wndToTFrame"]:FindChild("Backer"):SetSprite("")
			end
			if unitFramesAddon.luaFocusFrame then
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetSprite("")
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("CastingBase"):SetSprite("")
				unitFramesAddon.luaFocusFrame["wndToTFrame"]:FindChild("Backer"):SetSprite("")
			end
		end	
		showBackground = false
	end
	-- Show simple HP
	if subcommand == "simplehp on" or subcommand == "shp on" then
		simpleHP = true
	end
	-- Hide simple HP
	if subcommand == "simplehp off" or subcommand == "shp off" then
		simpleHP = false
	end
	-- Animated Model On
	if subcommand == "animatedmodel on" or animatedModel == true then
		animatedModel = true
		if unitFramesAddon.luaUnitFrame then
			unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetModel"):SetAnimated(animatedModel)
		end
		if unitFramesAddon.luaTargetFrame then
			unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetModel"):SetAnimated(animatedModel)
			unitFramesAddon.luaTargetFrame["wndToTFrame"]:FindChild("TargetModel"):SetAnimated(animatedModel)
		end
		if unitFramesAddon.luaFocusFrame then
			unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetModel"):SetAnimated(animatedModel)
			unitFramesAddon.luaFocusFrame["wndToTFrame"]:FindChild("TargetModel"):SetAnimated(animatedModel)
		end
	end
	-- Animated Model Off
	if subcommand == "animatedmodel off" or animatedModel == false then
		animatedModel = false
		if unitFramesAddon.luaUnitFrame then
			unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetModel"):SetAnimated(animatedModel)
		end
		if unitFramesAddon.luaTargetFrame then
			unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetModel"):SetAnimated(animatedModel)
			unitFramesAddon.luaTargetFrame["wndToTFrame"]:FindChild("TargetModel"):SetAnimated(animatedModel)
		end
		if unitFramesAddon.luaFocusFrame then
			unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetModel"):SetAnimated(animatedModel)
			unitFramesAddon.luaFocusFrame["wndToTFrame"]:FindChild("TargetModel"):SetAnimated(animatedModel)
		end
	end
	
	-- Show Total Percentage
	if subcommand == "percentage on" or subcommand == "% on" then
		showTotalPercentage = true
	end
	-- Hide Total Percentage
	if subcommand == "percentage off" or subcommand == "% off" then
		showTotalPercentage = false
	end
	
	-- Show Level
	if subcommand == "level show" or subcommand == "lvl s" or showLevel == true then
		if unitFramesAddon then
			if unitFramesAddon.luaUnitFrame then
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetLevel"):Show(true)
			end
			if unitFramesAddon.luaTargetFrame then
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetLevel"):Show(true)
			end
			if unitFramesAddon.luaFocusFrame then
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetLevel"):Show(true)
			end
		end	
		showLevel = true
	end
	
	-- Hide Level
	if subcommand == "level hide" or subcommand == "lvl h" or showLevel == false then
		if unitFramesAddon then
			if unitFramesAddon.luaUnitFrame then
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetLevel"):Show(false)
			end
			if unitFramesAddon.luaTargetFrame then
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetLevel"):Show(false)
			end
			if unitFramesAddon.luaFocusFrame then
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetLevel"):Show(false)
			end
		end	
		showLevel = false
	end
	
	-- Show Class
	if subcommand == "class show" or subcommand == "class s" or showClass == true then
		if unitFramesAddon then
			if unitFramesAddon.luaUnitFrame then
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetIconShadow"):SetSprite("HUD_TargetFrame:spr_TargetFrame_IconShadow")
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetClassIcon"):Show(true)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("PlayerClassIcon"):Show(true)
			end
			if unitFramesAddon.luaTargetFrame then
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetIconShadow"):SetSprite("HUD_TargetFrame:spr_TargetFrame_IconShadow")
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetClassIcon"):Show(true)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("PlayerClassIcon"):Show(true)
			end
			if unitFramesAddon.luaFocusFrame then
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetIconShadow"):SetSprite("HUD_TargetFrame:spr_TargetFrame_IconShadow")
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetClassIcon"):Show(true)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("PlayerClassIcon"):Show(true)
			end
		end	
		showClass = true
	end
	
	-- Hide Class
	if subcommand == "class hide" or subcommand == "class h" or showClass == false then
		if unitFramesAddon then
			if unitFramesAddon.luaUnitFrame then
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetIconShadow"):SetSprite("")
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetClassIcon"):Show(false)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("PlayerClassIcon"):Show(false)
			end
			if unitFramesAddon.luaTargetFrame then
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetIconShadow"):SetSprite("")
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetClassIcon"):Show(false)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("PlayerClassIcon"):Show(false)
			end
			if unitFramesAddon.luaFocusFrame then
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetIconShadow"):SetSprite("")
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetClassIcon"):Show(false)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("PlayerClassIcon"):Show(false)
			end
		end	
		showClass = false
	end
	
	if subcommand == "" then
		Print("Slim UnitFrame Settings")
		self.wndSettings:Invoke()
	end
	
	-- Hide alternative font
	if subcommand == "font show" or subcommand == "f s" or alternativeFont == true then
		if unitFramesAddon then
			if unitFramesAddon.luaUnitFrame then
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetName"):SetFont("CRB_Interface10_BO")
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("HealthText"):SetFont("CRB_Interface9_BO")
			end
			if unitFramesAddon.luaTargetFrame then
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetName"):SetFont("CRB_Interface10_BO")
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("HealthText"):SetFont("CRB_Interface9_BO")
			end
			if unitFramesAddon.luaFocusFrame then
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetName"):SetFont("CRB_Interface10_BO")
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("HealthText"):SetFont("CRB_Interface9_BO")
			end
		end	
		alternativeFont = true
	end
	
	-- Hide alternative font
	if subcommand == "font hide" or subcommand == "f h" or alternativeFont == false then
		if unitFramesAddon then
			if unitFramesAddon.luaUnitFrame then
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetName"):SetFont("Nameplates")
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("HealthText"):SetFont("CRB_Pixel_O")
			end
			if unitFramesAddon.luaTargetFrame then
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetName"):SetFont("Nameplates")
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("HealthText"):SetFont("CRB_Pixel_O")
			end
			if unitFramesAddon.luaFocusFrame then
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetName"):SetFont("Nameplates")
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("HealthText"):SetFont("CRB_Pixel_O")
			end
		end	
		alternativeFont = false
	end
	
	-- Fade out combat
	if fadeOutCombat == false then
		if unitFramesAddon then
			if unitFramesAddon.luaUnitFrame then
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:SetOpacity(1)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetModel"):SetOpacity(1)
			end
			if unitFramesAddon.luaTargetFrame then
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:SetOpacity(1)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetModel"):SetOpacity(1)
			end
			if unitFramesAddon.luaFocusFrame then
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:SetOpacity(1)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetModel"):SetOpacity(1)
			end
		end	
	end
	
end

function SlimUnitFrame:OnTimer()
	local unitPlayer = GameLib.GetPlayerUnit()
	local bInCombat
	if unitPlayer ~= nil then
		bInCombat = unitPlayer:IsInCombat()
	end	
	if runOnce then
		if unitFramesAddon then
			if unitFramesAddon.luaUnitFrame then
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("CastingFrame"):SetAnchorOffsets(104, 60, 283, 109)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("CastingName"):SetAnchorOffsets(16, -18, -16, 19)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("CastingBase"):SetSprite("BK3:sprGradientOutwards")
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("CastingBase"):SetBGColor("UI_AlphaPercent25")
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("CastingBase"):SetAnchorOffsets(0, 18, 0, 0)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetName"):SetAnchorOffsets(134, 14, 363, 34)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetSprite("BK3:sprGradientOutwards")
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetBGColor("UI_AlphaPercent25")
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetAnchorOffsets(93, 17, -62, -26)				
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("HealthSplit"):SetAnchorOffsets(132, 16, 191, 52)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("MaxHealth"):SetAnchorOffsets(0, 9, 174, 9)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("MaxShield"):SetAnchorOffsets(136, 9, 210, 9)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("MaxAbsorb"):SetAnchorOffsets(166, 9, 240, 9)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("HealthText"):SetAnchorOffsets(36, 5, 211, 5)				
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("Window"):SetAnchorOffsets(54, 0, 118, 64)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetModel"):SetAnchorOffsets(56, 0, 120, 64)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetModel"):SetSprite("ClientSprites:WhiteFill")
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetModel"):SetBGColor("UI_AlphaPercent25")
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("GroupSizeMark"):SetAnchorOffsets(41, 40, -66, 62)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("GroupSizeMark"):SetSprite("")
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("RaidMarker"):SetAnchorOffsets(73, -34, 105, -2)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetLevel"):SetAnchorOffsets(35, 47, 77, 89)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("BeneBuffBar"):SetAnchorOffsets(120, -20, 370, 10)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("HarmBuffBar"):SetAnchorOffsets(120, -50, 370, -20)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("CCArmor"):SetAnchorOffsets(-86, 13, -26, 84)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("Backer"):Show(false)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetScalingMark"):SetAnchorOffsets(8, 36, 30, 54)
				unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetGoalPanel"):SetAnchorOffsets(20, 4, 44, 28)
				--unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("FriendMark"):SetAnchorOffsets(40, 2, 65, 32)
				--unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("FriendMarkIcon"):SetAnchorOffsets(40, 2, 65, 32)
				--unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("FriendMark"):Show(false)
				--unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("FriendMarkIcon"):Show(false)
				ok = ok + 1
			end

			if unitFramesAddon.luaTargetFrame then
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("CastingFrame"):SetAnchorOffsets(-300, 60, -104, 109)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("CastingBase"):SetSprite("BK3:sprGradientOutwards")		
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("CastingBase"):SetBGColor("UI_AlphaPercent25")
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("CastingName"):SetAnchorOffsets(6, -18, -26, 19)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("CastingBase"):SetAnchorOffsets(0, 18, 0, 0)				
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetSprite("BK3:sprGradientOutwards")
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetBGColor("UI_AlphaPercent25")
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetAnchorOffsets(71, 17, -84, -26)				
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("HealthSplit"):SetAnchorOffsets(120, 16, 179, 52)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("MaxHealth"):SetAnchorOffsets(0, 10, 174, 10)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("MaxShield"):SetAnchorOffsets(135, 10, 209, 10)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("MaxAbsorb"):SetAnchorOffsets(165, 10, 239, 10)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("HealthText"):SetAnchorOffsets(36, 5, 211, 5)				
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetName"):SetAnchorOffsets(-363, 14, -134, 34)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("Window"):SetAnchorOffsets(-122, 0, -58, 64)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetModel"):SetAnchorOffsets(-120, 0, -56, 64)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("GroupSizeMark"):SetAnchorOffsets(-66, 40, -41, 62)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("GroupSizeMark"):SetSprite("")
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("RaidMarker"):SetAnchorOffsets(-105, -34, -73, -2)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetLevel"):SetAnchorOffsets(-77, 47, -35, 89)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("BeneBuffBar"):SetAnchorOffsets(-370, -20, -120, 10)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("HarmBuffBar"):SetAnchorOffsets(-370, -50, -120, -20)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("CCArmor"):SetAnchorOffsets(26, 13, 86, 84)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("Backer"):Show(false)		
				unitFramesAddon.luaTargetFrame["wndToTFrame"]:FindChild("Backer"):SetSprite("BK3:sprGradientOutwards")
				unitFramesAddon.luaTargetFrame["wndToTFrame"]:FindChild("Backer"):SetBGColor("UI_AlphaPercent25")
				unitFramesAddon.luaTargetFrame["wndToTFrame"]:FindChild("Backer"):SetScale(0.7)
				unitFramesAddon.luaTargetFrame["wndToTFrame"]:FindChild("Backer"):SetAnchorOffsets(-7, 10, -7, 10)
				unitFramesAddon.luaTargetFrame["wndToTFrame"]:FindChild("TargetModel"):SetAnchorOffsets(-2, 12, 34, 48) --Careful, it's movement range is limited			
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetScalingMark"):SetAnchorOffsets(-30, 36, -8, 54)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetGoalPanel"):SetAnchorOffsets(-44, 4, -20, 28)
				--unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("FriendMark"):SetAnchorOffsets(-65, 2, 40, 32)
				--unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("FriendMarkIcon"):SetAnchorOffsets(-65, 2, 40, 32)
				--unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("FriendMark"):Show(false)
				--unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("FriendMarkIcon"):Show(false)
				unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("PetContainer"):SetAnchorOffsets(-133, 14, -111, 36)
				ok = ok + 1
			end	
			
			if unitFramesAddon.luaFocusFrame then
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("CastingFrame"):SetAnchorOffsets(104, 60, 283, 109)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("CastingName"):SetAnchorOffsets(16, -18, -16, 19)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("CastingBase"):SetSprite("BK3:sprGradientOutwards")		
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("CastingBase"):SetBGColor("UI_AlphaPercent25")
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("CastingBase"):SetAnchorOffsets(0, 18, 0, 0)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetName"):SetAnchorOffsets(134, 14, 363, 34)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetSprite("BK3:sprGradientOutwards")
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetBGColor("UI_AlphaPercent25")
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("LargeBarContainer"):SetAnchorOffsets(93, 17, -62, -26)				
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("HealthSplit"):SetAnchorOffsets(132, 16, 191, 52)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("MaxHealth"):SetAnchorOffsets(0, 9, 174, 9)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("MaxShield"):SetAnchorOffsets(136, 9, 210, 9)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("MaxAbsorb"):SetAnchorOffsets(166, 9, 240, 9)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("HealthText"):SetAnchorOffsets(36, 5, 211, 5)	
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("Window"):SetAnchorOffsets(54, 0, 118, 64)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetModel"):SetAnchorOffsets(56, 0, 120, 64)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("GroupSizeMark"):SetAnchorOffsets(41, 40, -66, 62)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("GroupSizeMark"):SetSprite("")
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("RaidMarker"):SetAnchorOffsets(73, -34, 105, -2)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetLevel"):SetAnchorOffsets(35, 47, 77, 89)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("BeneBuffBar"):SetAnchorOffsets(120, -20, 370, 10)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("HarmBuffBar"):SetAnchorOffsets(120, -50, 370, -20)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("CCArmor"):SetAnchorOffsets(-86, 13, -26, 84)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("Backer"):Show(false)			
				unitFramesAddon.luaFocusFrame["wndToTFrame"]:FindChild("Backer"):SetSprite("BK3:sprGradientOutwards")
				unitFramesAddon.luaFocusFrame["wndToTFrame"]:FindChild("Backer"):SetBGColor("UI_AlphaPercent25")
				unitFramesAddon.luaFocusFrame["wndToTFrame"]:FindChild("Backer"):SetScale(0.7)
				unitFramesAddon.luaFocusFrame["wndToTFrame"]:FindChild("Backer"):SetAnchorOffsets(12, 8, 12, 8)
				unitFramesAddon.luaFocusFrame["wndToTFrame"]:FindChild("TargetModel"):SetAnchorOffsets(20, 10, 56, 46) --Careful, it's movement range is limited
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetScalingMark"):SetAnchorOffsets(-8, 36, -30, 54)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetGoalPanel"):SetAnchorOffsets(20, 4, 44, 28)
				--unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("FriendMark"):SetAnchorOffsets(40, 2, 65, 32)
				--unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("FriendMarkIcon"):SetAnchorOffsets(40, 2, 65, 32)
				--unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("FriendMark"):Show(false)
				--unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("FriendMarkIcon"):Show(false)
				unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("PetContainer"):SetAnchorOffsets(111, 14, 133, 36)
				
				ok = ok + 1
			end	
			self.onSlashCommand("","")
		end		
		if ok == 3 then
			runOnce = false
		end	
		ok = 0
	end
	
	-- Fade Frames out of combat
	if fadeOutCombat then
		if bInCombat then
			if unitFramesAddon then
				if unitFramesAddon.luaUnitFrame then
					unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetModel"):SetOpacity(1.0)
					unitFramesAddon.luaUnitFrame["wndLargeFrame"]:SetOpacity(1.0)
				end
				if unitFramesAddon.luaTargetFrame then
					unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetModel"):SetOpacity(1.0)
					unitFramesAddon.luaTargetFrame["wndToTFrame"]:FindChild("TargetModel"):SetOpacity(1.0)
					unitFramesAddon.luaTargetFrame["wndLargeFrame"]:SetOpacity(1.0)
				end
				if unitFramesAddon.luaFocusFrame then
					unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetModel"):SetOpacity(1.0)
					unitFramesAddon.luaFocusFrame["wndLargeFrame"]:SetOpacity(1.0)
				end
			end	
		else
			if unitFramesAddon then
				if unitFramesAddon.luaUnitFrame then
					unitFramesAddon.luaUnitFrame["wndLargeFrame"]:FindChild("TargetModel"):SetOpacity(0.7)
					unitFramesAddon.luaUnitFrame["wndLargeFrame"]:SetOpacity(0.7)
				end
				if unitFramesAddon.luaTargetFrame then
					unitFramesAddon.luaTargetFrame["wndLargeFrame"]:FindChild("TargetModel"):SetOpacity(0.7)
					unitFramesAddon.luaTargetFrame["wndToTFrame"]:FindChild("TargetModel"):SetOpacity(0.7)
					unitFramesAddon.luaTargetFrame["wndLargeFrame"]:SetOpacity(0.7)
				end
				if unitFramesAddon.luaFocusFrame then
					unitFramesAddon.luaFocusFrame["wndLargeFrame"]:FindChild("TargetModel"):SetOpacity(0.7)
					unitFramesAddon.luaFocusFrame["wndLargeFrame"]:SetOpacity(0.7)
				end
			end	
		end
	end	
end		

function SlimUnitFrame:ArrangeClusterMembers()
	local nFrameLeft, nFrameTop, nFrameRight, nFrameBottom = self.arClusterFrames[1]:GetRect()
	local unitPlayer = GameLib.GetPlayerUnit()
	local bInCombat
	if unitPlayer ~= nil then
		bInCombat = unitPlayer:IsInCombat()
	end	
	
	if fadeOutCombat then
		if bInCombat then		
			self.arClusterFrames[2]:FindChild("TargetModel"):SetOpacity(1)
			self.arClusterFrames[2]:SetOpacity(1)
			self.arClusterFrames[3]:FindChild("TargetModel"):SetOpacity(1)
			self.arClusterFrames[3]:SetOpacity(1)
			self.arClusterFrames[4]:FindChild("TargetModel"):SetOpacity(1)
			self.arClusterFrames[4]:SetOpacity(1)
			self.arClusterFrames[5]:FindChild("TargetModel"):SetOpacity(1)
			self.arClusterFrames[5]:SetOpacity(1)
		else
			self.arClusterFrames[2]:FindChild("TargetModel"):SetOpacity(0.7)
			self.arClusterFrames[2]:SetOpacity(0.7)
			self.arClusterFrames[3]:FindChild("TargetModel"):SetOpacity(0.7)
			self.arClusterFrames[3]:SetOpacity(0.7)
			self.arClusterFrames[4]:FindChild("TargetModel"):SetOpacity(0.7)
			self.arClusterFrames[4]:SetOpacity(0.7)
			self.arClusterFrames[5]:FindChild("TargetModel"):SetOpacity(0.7)
			self.arClusterFrames[5]:SetOpacity(0.7)
		end
	else
		self.arClusterFrames[2]:SetOpacity(1)
		self.arClusterFrames[3]:SetOpacity(1)
		self.arClusterFrames[4]:SetOpacity(1)
		self.arClusterFrames[5]:SetOpacity(1)	
	end	

	if self.nFrameLeft == nil or nFrameLeft ~= self.nFrameLeft or nFrameTop ~= self.nFrameTop then -- if the frame has been moved since we last drew
		-- set new variables
		self.nFrameLeft = nFrameLeft + 50
		self.nFrameTop = nFrameTop
		self.nFrameRight = nFrameRight - 50
		self.nFrameBottom = nFrameBottom
		
		--self.arClusterFrames[1]:FindChild("TargetModel"):SetAnimated(false) -- [1] is not a pet unit

		self.arClusterFrames[2]:FindChild("TargetModel"):SetAnimated(animatedModel)
		self.arClusterFrames[2]:FindChild("ClusterTargetBG"):SetSprite("")
		self.arClusterFrames[2]:SetScale(0.9)
		self.arClusterFrames[3]:FindChild("TargetModel"):SetAnimated(animatedModel)
		self.arClusterFrames[3]:FindChild("ClusterTargetBG"):SetSprite("")
		self.arClusterFrames[3]:SetScale(0.9)
		self.arClusterFrames[4]:FindChild("TargetModel"):SetAnimated(animatedModel)
		self.arClusterFrames[4]:FindChild("ClusterTargetBG"):SetSprite("")
		self.arClusterFrames[4]:SetScale(0.9)
		self.arClusterFrames[5]:FindChild("TargetModel"):SetAnimated(animatedModel)
		self.arClusterFrames[5]:FindChild("ClusterTargetBG"):SetSprite("")
		self.arClusterFrames[5]:SetScale(0.9)
		
		if self.tParams.bFlipped then
			self.arClusterFrames[2]:Move(self.nFrameRight-50*1, self.nFrameTop + 120, 60, 62)
			self.arClusterFrames[3]:Move(self.nFrameRight-50*2, self.nFrameTop + 120, 60, 62)
			self.arClusterFrames[4]:Move(self.nFrameRight-50*3, self.nFrameTop + 120, 60, 62)
			self.arClusterFrames[5]:Move(self.nFrameRight-50*4, self.nFrameTop + 120, 60, 62)
		else
			self.arClusterFrames[2]:Move(self.nFrameLeft+50*0, self.nFrameTop + 120, 60, 62)
			self.arClusterFrames[3]:Move(self.nFrameLeft+50*1, self.nFrameTop + 120, 60, 62)
			self.arClusterFrames[4]:Move(self.nFrameLeft+50*2, self.nFrameTop + 120, 60, 62)
			self.arClusterFrames[5]:Move(self.nFrameLeft+50*3, self.nFrameTop + 120, 60, 62)
		end
	end
end

function SlimUnitFrame:UpdateCastingBar(wndFrame, unitCaster)
	-- Casting Bar Update

	local bShowCasting = false
	local bEnableGlow = false
	local nZone = 0
	local nMaxZone = 0
	local nDuration = 0
	local nElapsed = 0
	local strSpellName = ""
	local nElapsed = 0
	local eType = Unit.CodeEnumCastBarType.None
	local strIcon = ""
	local strFillSprite = ""
	local strBaseSprite = ""
	local strGlowSprite = ""

	local wndCastFrame = wndFrame:FindChild("CastingFrame")
	local wndCastProgress = wndFrame:FindChild("CastingBar")
	local wndCastName = wndFrame:FindChild("CastingName")
	local wndCastIcon = wndFrame:FindChild("CastingIcon")
	local wndCastBase = wndFrame:FindChild("CastingBase")

	-- results for GetCastBarType can be:
	-- Unit.CodeEnumCastBarType.None
	-- Unit.CodeEnumCastBarType.Normal
	-- Unit.CodeEnumCastBarType.Telegraph_Backlash
	-- Unit.CodeEnumCastBarType.Telegraph_Evade
	if unitCaster:ShouldShowCastBar() then
		eType = unitCaster:GetCastBarType()

		if eType == Unit.CodeEnumCastBarType.Telegraph_Evade then
			strIcon = "CRB_TargetFrameSprites:sprTF_CastIconEvade"
		elseif eType == Unit.CodeEnumCastBarType.Telegraph_Backlash then
			strIcon = "CRB_TargetFrameSprites:sprTF_CastIconInterrupt"
		else
			strIcon = ""
		end

		if eType ~= Unit.CodeEnumCastBarType.None then

			bShowCasting = true
			bEnableGlow = true
			nZone = 0
			nMaxZone = 1
			nDuration = unitCaster:GetCastDuration()
			nElapsed = unitCaster:GetCastElapsed()
			if wndCastProgress ~= nil then
				wndCastProgress:SetTickLocations(0, 100, 200, 300)
			end

			strSpellName = unitCaster:GetCastName()
		end
	end

	wndCastFrame:Show(bShowCasting)
	if wndCastProgress ~= nil then
		wndCastProgress:Show(bShowCasting)
		wndCastName:Show(bShowCasting)
	end

	if bShowCasting and nDuration > 0 and nMaxZone > 0 then
		wndCastIcon:SetSprite(nIcon)

		if wndCastProgress ~= nil then
			-- add a countdown timer if nDuration is > 0 seconds. ' Changed
			local strDuration = nDuration > 0 and " (" .. string.format("%00.01f", (nDuration-nElapsed)/1000)..")" or ""
			
			wndCastProgress:Show(bShowCasting)
			wndCastProgress:SetMax(nDuration)
			wndCastProgress:SetProgress(nElapsed)
			wndCastProgress:EnableGlow(bEnableGlow)
			wndCastName:SetText(strSpellName .. strDuration)
		end
	end
end


function SlimUnitFrame:SetTargetHealthAndShields(wndTargetFrame, unitTarget)
	if not unitTarget or unitTarget:GetHealth() == nil then
		return
	end

	if unitTarget:GetType() == "Simple" then -- String Comparison, should replace with an enum
		self.wndLargeFrame:FindChild("HealthText"):SetText("")
		self.wndLargeFrame:FindChild("MaxShield"):Show(false)
		self.wndLargeFrame:FindChild("MaxAbsorb"):Show(false)
		return
	end

	local nHealthCurr = unitTarget:GetHealth()
	local nHealthMax = unitTarget:GetMaxHealth()
	local nShieldCurr = unitTarget:GetShieldCapacity()
	local nShieldMax = unitTarget:GetShieldCapacityMax()
	local nAbsorbCurr = 0
	local nAbsorbMax = unitTarget:GetAbsorptionMax()
	if nAbsorbMax > 0 then
		nAbsorbCurr = unitTarget:GetAbsorptionValue() -- Since it doesn't clear when the buff drops off
	end
	local nTotalMax = nHealthMax + nShieldMax + nAbsorbMax
	
	local strFlipped = self.tParams.bFlipped and "Flipped" or ""
	local wndHealth =  self.wndLargeFrame:FindChild("HealthCapacityTint")
	if unitTarget:IsInCCState(Unit.CodeEnumCCState.Vulnerability) then
		wndHealth:SetFullSprite("spr_TargetFrame_HealthFillVulernable"..strFlipped)
	elseif nHealthCurr / nHealthMax <= knHealthRed then
		wndHealth:SetFullSprite("spr_TargetFrame_HealthFillRed"..strFlipped)
	elseif nHealthCurr / nHealthMax <= knHealthYellow then
		wndHealth:SetFullSprite("spr_TargetFrame_HealthFillYellow"..strFlipped)
	else
		wndHealth:SetFullSprite("spr_TargetFrame_HealthFillGreen"..strFlipped)
	end
	
	wndHealth:SetStyleEx("EdgeGlow", nHealthCurr / nHealthMax < 0.96)

	-- Resize
	self:SetBarValue(self.wndLargeFrame:FindChild("ShieldCapacityTint"), 0, nShieldCurr, nShieldMax) -- Only the Curr Shield really progress fills
	self:SetBarValue(self.wndLargeFrame:FindChild("AbsorbCapacityTint"), 0, nAbsorbCurr, nAbsorbMax)

	-- Bars
	self.wndLargeFrame:FindChild("HealthCapacityTint"):SetMax(nHealthMax);
	self.wndLargeFrame:FindChild("HealthCapacityTint"):SetProgress(nHealthCurr);
	
	self.wndLargeFrame:FindChild("ShieldCapacityTint"):SetMax(nShieldMax);
	self.wndLargeFrame:FindChild("ShieldCapacityTint"):SetProgress(nShieldCurr);
	
	self.wndLargeFrame:FindChild("AbsorbCapacityTint"):SetMax(nAbsorbMax);
	self.wndLargeFrame:FindChild("AbsorbCapacityTint"):SetProgress(nAbsorbCurr);
	
	self.wndLargeFrame:FindChild("MaxShield"):Show(nShieldCurr > 0 and nShieldMax > 0)-- and unitTarget:ShouldShowShieldCapacityBar())
	self.wndLargeFrame:FindChild("MaxAbsorb"):Show(nAbsorbCurr > 0 and nAbsorbMax > 0)-- and unitTarget:ShouldShowShieldCapacityBar())
	--self.wndLargeFrame:FindChild("MaxAbsorb"):MoveToLocation(self.wndLargeFrame:FindChild("MaxShield"):IsShown() and self.arAbsorbPos or self.arShieldPos)

	if not self.wndLargeFrame:FindChild("MaxShield"):IsShown() and not self.wndLargeFrame:FindChild("MaxAbsorb"):IsShown() then
		-- reduce by 2
		if self.tParams.bFlipped then
			self.wndLargeFrame:SetAnchorOffsets(self.nLFrameRight-knFrameWidthMin, self.nLFrameTop, self.nLFrameRight, self.nLFrameBottom)
		else
			self.wndLargeFrame:SetAnchorOffsets(self.nLFrameLeft, self.nLFrameTop, self.nLFrameLeft+knFrameWidthMin, self.nLFrameBottom)
		end
		self.wndLargeFrame:FindChild("HealthSplit"):Show(false)
	elseif not self.wndLargeFrame:FindChild("MaxShield"):IsShown() or not self.wndLargeFrame:FindChild("MaxAbsorb"):IsShown() then
		-- reduce by 1
		if self.tParams.bFlipped then
			self.wndLargeFrame:SetAnchorOffsets(self.nLFrameRight-knFrameWidthShield, self.nLFrameTop, self.nLFrameRight, self.nLFrameBottom)
		else
			self.wndLargeFrame:SetAnchorOffsets(self.nLFrameLeft, self.nLFrameTop, self.nLFrameLeft+knFrameWidthShield, self.nLFrameBottom)
		end
		self.wndLargeFrame:FindChild("HealthSplit"):Show(true)
	else
		if self.tParams.bFlipped then
			self.wndLargeFrame:SetAnchorOffsets(self.nLFrameRight-knFrameWidthMax, self.nLFrameTop, self.nLFrameRight, self.nLFrameBottom)
		else
			self.wndLargeFrame:SetAnchorOffsets(self.nLFrameLeft, self.nLFrameTop, self.nLFrameLeft+knFrameWidthMax, self.nLFrameBottom)
		end
		self.wndLargeFrame:FindChild("HealthSplit"):Show(true)
	end

	-- String
	local strHealthMax = self:HelperFormatBigNumber(nHealthMax)
	local strHealthCurr = self:HelperFormatBigNumber(nHealthCurr)
	local strShieldCurr = self:HelperFormatBigNumber(nShieldCurr)
	local strShieldMax = self:HelperFormatBigNumber(nShieldMax)
	local strAbsorbCurr = self:HelperFormatBigNumber(nAbsorbCurr)
	local strAbsorbMax = self:HelperFormatBigNumber(nAbsorbMax)
	
	-- Show Simple HP --
	local spHealthCurr = tostring(nHealthCurr)
	local spShieldCurr = tostring(nShieldCurr)
	local spAbsorbCurr = tostring(nAbsorbCurr)
	
	while true do  -- Add decimals do values
		spHealthCurr, k = string.gsub(spHealthCurr, "^(-?%d+)(%d%d%d)",'%1.%2')
		spShieldCurr, k = string.gsub(spShieldCurr, "^(-?%d+)(%d%d%d)",'%1.%2')
		spAbsorbCurr, k = string.gsub(spAbsorbCurr, "^(-?%d+)(%d%d%d)",'%1.%2')
		if (k==0) then
		  break
		end
	end
	---
	
	local nVisibility = Apollo.GetConsoleVariable("hud.healthTextDisplay")
	
	local strTextXY 
	if simpleHP == true then
		if nVisibility == 2 then
			if nHealthCurr > 999999 then
				if math.floor((spHealthCurr % 1000)/10) > 9 then
					strTextXY = math.floor(spHealthCurr / 1000).."."..math.floor((spHealthCurr % 1000)/10).."m"
				else
					strTextXY = math.floor(spHealthCurr / 1000)..".0"..math.floor((spHealthCurr % 1000)/10).."m"
				end	
			else
				strTextXY = spHealthCurr
			end
		end	
	else
		strTextXY = (nVisibility == 2) and String_GetWeaselString(Apollo.GetString("TargetFrame_HealthText"), strHealthCurr, strHealthMax) or ""
	end
	strTextPCT = (nVisibility == 3) and String_GetWeaselString(Apollo.GetString("CRB_Percent"), nHealthCurr/nHealthMax*100) or ""
	local strTooltip = string.format("%s: %s / %s (%s)", Apollo.GetString("Innate_Health"), strHealthCurr, strHealthMax, String_GetWeaselString(Apollo.GetString("CRB_Percent"), nHealthCurr/nHealthMax*100))
	
	if nShieldMax > 0 and nShieldCurr > 0 then
		if nVisibility == 2 then
			if simpleHP == true then
				if nShieldCurr > 999999 then
					if math.floor((spShieldCurr % 1000)/10) > 9 then
						strTextXY = strTextXY.." ("..math.floor(spShieldCurr / 1000).."."..math.floor((spShieldCurr % 1000)/10).."m)"
					else
						strTextXY = strTextXY.." ("..math.floor(spShieldCurr / 1000)..".0"..math.floor((spShieldCurr % 1000)/10).."m)"
					end	
				else
					strTextXY = strTextXY.." ("..spShieldCurr..")"
				end	
			else
				strTextXY = String_GetWeaselString(Apollo.GetString("TargetFrame_HealthShieldText"), strTextXY, strShieldCurr)
			end
		elseif nVisibility == 3 and nShieldCurr/nShieldMax ~= nHealthCurr/nHealthMax then
			strTextPCT = String_GetWeaselString(Apollo.GetString("TargetFrame_HealthShieldText"), strTextPCT, String_GetWeaselString(Apollo.GetString("CRB_Percent"), nShieldCurr/nShieldMax*100))
		end
		
		local strShieldTooltip = string.format("%s: %s / %s (%s)", Apollo.GetString("Character_ShieldLabel"), strShieldCurr, strShieldMax, String_GetWeaselString(Apollo.GetString("CRB_Percent"), nShieldCurr/nShieldMax*100))
		strTooltip = string.format("%s\n%s", strTooltip, strShieldTooltip)
	end
	
	if nAbsorbMax > 0 and nAbsorbCurr > 0 then
		local strAbsorbTooltip = string.format("%s: %s / %s (%s)", Apollo.GetString("FloatText_AbsorbTester"), strAbsorbCurr, strAbsorbMax, String_GetWeaselString(Apollo.GetString("CRB_Percent"), nAbsorbCurr/nAbsorbMax*100))
		strTooltip = string.format("%s\n%s", strTooltip, strAbsorbTooltip)
		
		if nVisibility == 2 then
			if nAbsorbCurr > 999999	then
				if math.floor((nAbsorbCurr % 1000)/10) > 9 then
					strTextXY = strTextXY.." ["..math.floor(spAbsorbCurr / 1000).."."..math.floor((spAbsorbCurr % 1000)/10).."m]"
				else
					strTextXY = strTextXY.." ["..math.floor(spAbsorbCurr / 1000)..".0"..math.floor((spAbsorbCurr % 1000)/10).."m]"	
				end	
			else
				strTextXY = strTextXY.." ["..spAbsorbCurr.."]"
			end	
		elseif nVisibility == 3 and nShieldCurr/nShieldMax ~= nHealthCurr/nHealthMax then
			--strTextPCT = strTextXY.." ["..(nAbsorbCurr/nAbsorbMax*100).."%]"
		end		
	end
	
	-- Show total percentage
	if nVisibility == 2 then
		if showTotalPercentage == true then
			local nTotalPercentage = math.floor((nHealthCurr + nShieldCurr + nAbsorbCurr)/(nHealthMax + nShieldMax + nAbsorbMax) * 100)
			if nTotalPercentage > 100 then
				nTotalPercentage = 100
			end
			strTextXY = strTextXY.." / "..nTotalPercentage.."%"
		end
	end
	
	--Toggle Visibility based on ui preference
	if nVisibility == 2 then -- show x/y
		self.wndLargeFrame:FindChild("HealthText"):SetText(strTextXY)
	elseif nVisibility == 3 then --show %
		self.wndLargeFrame:FindChild("HealthText"):SetText(strTextPCT)
	else --on mouseover
		self.wndLargeFrame:FindChild("HealthText"):SetText("")
	end
	
	self.wndLargeFrame:FindChild("HealthText"):SetTooltip(strTooltip)
end

function SlimUnitFrame:SetTargetForFrame(wndFrame, unitTarget, bTargetChanged)
	wndFrame:SetData(unitTarget)
	self:SetTargetHealthAndShields(wndFrame, unitTarget)

	if unitTarget then
		strName = unitTarget:GetName()
		-- PvP
		if unitTarget:IsPvpFlagged() then
			strName = String_GetWeaselString(Apollo.GetString("BaseBar_PvPAppend"), strName)
		end
		
		wndFrame:FindChild("TargetName"):SetText(strName)

		local eRank = unitTarget:GetRank()
		local strClassIconSprite = ""
		local strPlayerIconSprite = ""
		
		-- Class Icon is based on player class or NPC rank
		if unitTarget:GetType() == "Player" then
			strPlayerIconSprite = karClassToIcon[unitTarget:GetClassId()]
		elseif eRank == Unit.CodeEnumRank.Elite then
			strClassIconSprite = "spr_TargetFrame_ClassIcon_Elite"
		elseif eRank == Unit.CodeEnumRank.Superior then
			strClassIconSprite = "spr_TargetFrame_ClassIcon_Superior"
		elseif eRank == Unit.CodeEnumRank.Champion then
			strClassIconSprite = "spr_TargetFrame_ClassIcon_Champion"
		elseif eRank == Unit.CodeEnumRank.Standard then
			strClassIconSprite = "spr_TargetFrame_ClassIcon_Standard"
		elseif eRank == Unit.CodeEnumRank.Minion then
			strClassIconSprite = "spr_TargetFrame_ClassIcon_Minion"
		elseif eRank == Unit.CodeEnumRank.Fodder then
			strClassIconSprite = "spr_TargetFrame_ClassIcon_Fodder"
		end
		
		wndFrame:FindChild("TargetIconShadow"):Show(strPlayerIconSprite ~= "" or strClassIconSprite ~= "")
		wndFrame:FindChild("PlayerClassIcon"):SetSprite(strPlayerIconSprite)
		wndFrame:FindChild("TargetClassIcon"):SetSprite(strClassIconSprite)
		
		--Disposition/flags
		local strFlipped = self.tParams.bFlipped and "Flipped" or ""
		local eDisposition = unitTarget:GetDispositionTo(GameLib.GetPlayerUnit())
		local strDisposition = "Friendly"
		local strAttachment = ""

		wndFrame:FindChild("TargetName"):SetTextColor(karDispositionColors[eDisposition])
		
		if eDisposition == Unit.CodeEnumDisposition.Hostile then
			strDisposition = "Hostile"
		elseif eDisposition == Unit.CodeEnumDisposition.Neutral then
			strDisposition = "Neutral"
		end
		
		--Rare and Elite NPCs
		self.wndLargeFrame:FindChild("Rare"):SetSprite("")
		self.wndLargeFrame:FindChild("Elite"):SetSprite("")
		if unitTarget:IsRare() then
			self.wndLargeFrame:FindChild("Rare"):SetSprite("spr_TargetFrame_Frame"..strDisposition.."Rare"..strFlipped)
		end
		if unitTarget:IsElite() then
			self.wndLargeFrame:FindChild("Elite"):SetSprite("spr_TargetFrame_Frame"..strDisposition.."Elite"..strFlipped)
		end
		
		--Unit in Vehicle
		if self.unitTarget:IsInVehicle() then
			strAttachment = "Vehicle"
		end
		
		--Iconic and Max Level Players
		local idArchetype = unitTarget:GetArchetype() and unitTarget:GetArchetype().idArchetype or 0
		local strFaction = karFactionToString[unitTarget:GetFaction()] or ""
		local bPetClone = unitTarget:GetType() == "Pet" and unitTarget:GetName() == unitTarget:GetUnitOwner():GetName()
		
		if (idArchetype == Unit.CodeEnumArchetype.Iconic or ((unitTarget:GetType() == "Player" or bPetClone) and unitTarget:GetLevel() == knMaxLevel)) then
			strAttachment = eDisposition == Unit.CodeEnumDisposition.Friendly and "FriendlyIconic" or "HostileIconic"
			strAttachment = strAttachment .. strFaction
		end
		
		self.wndLargeFrame:FindChild("Attachment"):SetSprite("spr_TargetFrame_Frame"..strAttachment..strFlipped)
		self.wndLargeFrame:FindChild("Backer"):SetSprite("spr_TargetFrame_Frame"..strDisposition..strFlipped)
		self.wndLargeFrame:FindChild("HealthSplit"):SetSprite("spr_TargetFrame_HealthSplit"..strDisposition..strFlipped)
		
		if (unitTarget:IsDead() or (unitTarget:IsTagged() and not unitTarget:IsTaggedByMe() and not unitTarget:IsSoftKill())) then
			self.wndLargeFrame:FindChild("Backer"):SetSprite("spr_TargetFrame_FrameTapped"..strFlipped)
		end
		
		local unitToT = unitTarget:GetTarget()
		if unitToT ~= nil then
			local eToTDisposition = unitToT:GetDispositionTo(GameLib.GetPlayerUnit())
			
			if eToTDisposition == Unit.CodeEnumDisposition.Hostile then
				--self.wndToTFrame:FindChild("Backer"):SetSprite("BK3:sprGradientOutwards")
				--self.wndToTFrame:FindChild("DispositionFrame"):SetBGColor(ApolloColor.new("ffff6d66"))
			elseif eToTDisposition == Unit.CodeEnumDisposition.Neutral then
				--self.wndToTFrame:FindChild("Backer"):SetSprite("BK3:sprGradientOutwards")
				--self.wndToTFrame:FindChild("DispositionFrame"):SetBGColor(ApolloColor.new("fffaff66"))
				--self.wndToTFrame:FindChild("DispositionFrame"):SetBGColor("UI_AlphaPercent25")
			else
				--self.wndToTFrame:FindChild("Backer"):SetSprite("BK3:sprGradientOutwards")
				--self.wndToTFrame:FindChild("DispositionFrame"):SetBGColor(ApolloColor.new("ff00f7de"))
			end
			
			self.wndToTFrame:FindChild("DispositionFrame"):SetTooltip(ktDispositionToTTooltip[eToTDisposition])
		end
		
		-- Setting - On aggro
		local playerUnit = GameLib.GetPlayerUnit()
		if unitToT == playerUnit then
			self.wndToTFrame:FindChild("Backer"):SetScale(1)
			self.wndToTFrame:FindChild("Backer"):SetSprite("spr_TargetFrame_ToTHostile")
			self.wndToTFrame:FindChild("Backer"):SetBGColor("UI_AlphaPercent35")
			self.wndToTFrame:FindChild("Backer"):SetAnchorOffsets(-14, 2, -14, 2)

		else
			self.wndToTFrame:FindChild("Backer"):SetScale(0.7)
			self.wndToTFrame:FindChild("Backer"):SetSprite("BK3:sprGradientOutwards")
			self.wndToTFrame:FindChild("Backer"):SetBGColor("UI_AlphaPercent25")
			self.wndToTFrame:FindChild("Backer"):SetAnchorOffsets(-7, 10, -7, 10)
		end
		--
		
		--todo: Tooltips)
		local crColorToUse = karDispositionColors[eDisposition]

		-- Level / Diff
		local nLevel = unitTarget:GetLevel()
		if nLevel == nil then
			wndFrame:FindChild("TargetLevel"):SetText(Apollo.GetString("CRB__2"))
			wndFrame:FindChild("TargetLevel"):SetTextColor(karConInfo[1][2])
			wndFrame:FindChild("TargetLevel"):SetTooltip("")
		else
			wndFrame:FindChild("TargetLevel"):SetText(unitTarget:GetLevel())

			if unitTarget:IsScaled() then
				wndFrame:FindChild("TargetScalingMark"):Show(true)
				wndFrame:FindChild("TargetLevel"):SetTextColor(kcrScalingCColor)
				
				if unitTarget ~= GameLib.GetPlayerUnit() then
					strRewardFormatted = String_GetWeaselString(Apollo.GetString("TargetFrame_CreatureScales"), unitTarget:GetLevel())
					local strLevelTooltip = self:HelperBuildTooltip(strRewardFormatted, Apollo.GetString("Adaptive"), kcrScalingHex)
					wndFrame:FindChild("TargetLevel"):FindChild("TargetLevel"):SetTooltip(strLevelTooltip)
				end
			else
				wndFrame:FindChild("TargetScalingMark"):Show(false)
				local nCon = self:HelperCalculateConValue(unitTarget)
				wndFrame:FindChild("TargetLevel"):SetTextColor(karConInfo[nCon][2])
				
				if unitTarget ~= GameLib.GetPlayerUnit() then
					strRewardFormatted = String_GetWeaselString(Apollo.GetString("TargetFrame_TargetXPReward"), karConInfo[nCon][4])
					local strLevelTooltip = self:HelperBuildTooltip(strRewardFormatted, karConInfo[nCon][3], karConInfo[nCon][5])
					wndFrame:FindChild("TargetLevel"):FindChild("TargetLevel"):SetTooltip(strLevelTooltip)
				end
			end
		end

		local strUnitType = unitTarget:GetType()
		if strUnitType == "Player" or strUnitType == "Pet" or strUnitType == "Esper Pet" then
			local unitPlayer = unitTarget:GetUnitOwner() or unitTarget
			if eDisposition == Unit.CodeEnumDisposition.Friendly or unitPlayer:IsThePlayer() then
				if unitPlayer:IsPvpFlagged() then
					crColorToUse = kcrFlaggedFriendlyTextColor
				elseif unitPlayer:IsInYourGroup() then
					crColorToUse = kcrGroupTextColor
				else
					crColorToUse = kcrDefaultUnflaggedAllyTextColor
				end
			else
				local bIsUnitFlagged = unitPlayer:IsPvpFlagged()
				local bAmIFlagged = GameLib.IsPvpFlagged()
				if not bAmIFlagged and not bIsUnitFlagged then
					crColorToUse = kcrNeutralEnemyTextColor
				elseif (bAmIFlagged and not bIsUnitFlagged) or (not bAmIFlagged and bIsUnitFlagged) then
					crColorToUse = kcrAggressiveEnemyTextColor
				end
			end
			wndFrame:FindChild("GroupSizeMark"):Show(false)
			wndFrame:FindChild("TargetName"):SetTextColor(crColorToUse)
		else -- NPC
			wndFrame:FindChild("GroupSizeMark"):Show(unitTarget:GetGroupValue() > 0)
			wndFrame:FindChild("GroupSizeMark"):SetText(unitTarget:GetGroupValue())

			local strGroupTooltip = self:HelperBuildTooltip(String_GetWeaselString(Apollo.GetString("TargetFrame_GroupSize"), unitTarget:GetGroupValue()), String_GetWeaselString(Apollo.GetString("TargetFrame_Man"), unitTarget:GetGroupValue()))
			wndFrame:FindChild("GroupSizeMark"):SetTooltip(strGroupTooltip)
		end

		-- Interrupt Armor
		---------------------------------------------------------------------------
		local nCCArmorValue = unitTarget:GetInterruptArmorValue()
		local nCCArmorMax = unitTarget:GetInterruptArmorMax()
		local wndCCArmor = wndFrame:FindChild("CCArmor")

		if nCCArmorMax == -1 then -- impervious
			wndCCArmor:Show(true)
			wndCCArmor:SetText("")
			wndCCArmor:SetSprite("spr_TargetFrame_InterruptArmor_Infinite")
		elseif nCCArmorMax > 0 then -- has armor, has value
			wndCCArmor:Show(true)
			wndCCArmor:SetText(nCCArmorValue)
			wndCCArmor:SetSprite("spr_TargetFrame_InterruptArmor_Value")
		else
			wndCCArmor:Show(false)
		end
	end

	wndFrame:FindChild("BeneBuffBar"):SetUnit(unitTarget)
	wndFrame:FindChild("HarmBuffBar"):SetUnit(unitTarget)

	self:UpdateCastingBar(wndFrame, unitTarget)
end

---------------------------------------------------------------------------------------------------
-- Options
---------------------------------------------------------------------------------------------------

function SlimUnitFrame:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return nil
	end

	local tSaved =
	{
		tshowPortraits = showPortraits,
		tsimpleHP = simpleHP,
		tshowBackground = showBackground,
		tshowLevel = showLevel,
		tshowTotalPercentage = showTotalPercentage,
		talternativeFont = alternativeFont,
		tshowClass = showClass,
		tanimatedModel = animatedModel,
		tfadeOutCombat = fadeOutCombat,
	}
	return tSaved
end

function SlimUnitFrame:OnRestore(eType, tSavedData)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character and tSavedData then
		if tSavedData.tsimpleHP ~= nil then
			simpleHP = tSavedData.tsimpleHP
		end
		if tSavedData.tshowPortraits ~= nil then
			showPortraits = tSavedData.tshowPortraits
		end
		if tSavedData.tshowBackground ~= nil then
			showBackground = tSavedData.tshowBackground
		end
		if tSavedData.tshowLevel ~= nil then
			showLevel = tSavedData.tshowLevel
		end
		if tSavedData.tshowTotalPercentage ~= nil then
			showTotalPercentage = tSavedData.tshowTotalPercentage
		end
		if tSavedData.talternativeFont ~= nil then
			alternativeFont = tSavedData.talternativeFont
		end	
		if tSavedData.tshowClass ~= nil then
			showClass = tSavedData.tshowClass
		end	
		if tSavedData.tanimatedModel ~= nil then
			animatedModel = tSavedData.tanimatedModel
		end	
		if tSavedData.tfadeOutCombat ~= nil then
			fadeOutCombat = tSavedData.tfadeOutCombat
		end	
	end
	self.onRestoreCalled = true
	self:UpdateSettings()
end


---------------------------------------------------------------------------------------------------
-- SlimUnitFrame Settings Functions
---------------------------------------------------------------------------------------------------
function SlimUnitFrame:OnClose()
	self.wndSettings:Close() -- hide the window
end


function SlimUnitFrame:ShowPortraitsOnCheck( wndHandler, wndControl, eMouseButton )
	showPortraits = true
	self.onSlashCommand("","")
end

function SlimUnitFrame:ShowPortraitsOnUnCheck( wndHandler, wndControl, eMouseButton )
	showPortraits = false
	self.onSlashCommand("","")
end

function SlimUnitFrame:ShowBackgroundOnCheck( wndHandler, wndControl, eMouseButton )
	showBackground = true
	self.onSlashCommand("","")
end

function SlimUnitFrame:ShowBackgroundOnUnCheck( wndHandler, wndControl, eMouseButton )
	showBackground = false
	self.onSlashCommand("","")
end

function SlimUnitFrame:ShowLevelOnCheck( wndHandler, wndControl, eMouseButton )
	showLevel = true
	self.onSlashCommand("","")
end

function SlimUnitFrame:ShowLevelOnUnCheck( wndHandler, wndControl, eMouseButton )
	showLevel = false
	self.onSlashCommand("","")
end

function SlimUnitFrame:SimpleHPOnCheck( wndHandler, wndControl, eMouseButton )
	simpleHP = true
end

function SlimUnitFrame:SimpleHPOnUnCheck( wndHandler, wndControl, eMouseButton )
	simpleHP = false
end

function SlimUnitFrame:ShowTotalPercentageOnCheck( wndHandler, wndControl, eMouseButton )
	showTotalPercentage = true
end

function SlimUnitFrame:ShowTotalPercentageOnUnCheck( wndHandler, wndControl, eMouseButton )
	showTotalPercentage = false
end

function SlimUnitFrame:ShowAlternativeFontOnCheck( wndHandler, wndControl, eMouseButton )
	alternativeFont = true
	self.onSlashCommand("","")
end

function SlimUnitFrame:ShowAlternativeFontOnUnCheck( wndHandler, wndControl, eMouseButton )
	alternativeFont = false
	self.onSlashCommand("","")
end

function SlimUnitFrame:ShowClassOnCheck( wndHandler, wndControl, eMouseButton )
	showClass = true
	self.onSlashCommand("","")
end

function SlimUnitFrame:ShowClassOnUnCheck( wndHandler, wndControl, eMouseButton )
	showClass = false
	self.onSlashCommand("","")
end

function SlimUnitFrame:AnimatedModelOnCheck( wndHandler, wndControl, eMouseButton )
	animatedModel = true
	self.onSlashCommand("","")
end

function SlimUnitFrame:AnimatedModelUnCheck( wndHandler, wndControl, eMouseButton )
	animatedModel = false
	self.onSlashCommand("","")
end

function SlimUnitFrame:FadeOutCombatOnCheck( wndHandler, wndControl, eMouseButton )
	fadeOutCombat = true
	self.onSlashCommand("","")
end

function SlimUnitFrame:FadeOutCombatUnCheck( wndHandler, wndControl, eMouseButton )
	fadeOutCombat = false
	self.onSlashCommand("","")
end

function SlimUnitFrame:UpdateSettings()
	if not self.onRestoreCalled or not self.onXmlDocLoadedCalled then
		return
	end
	
	self.wndSettings:FindChild("ShowPortraitsBtn"):SetCheck(showPortraits)
	self.wndSettings:FindChild("ShowBackgroundBtn"):SetCheck(showBackground)
	self.wndSettings:FindChild("ShowLevelBtn"):SetCheck(showLevel)
	self.wndSettings:FindChild("ShowSimpleHPBtn"):SetCheck(simpleHP)
	self.wndSettings:FindChild("ShowTotalPercentageBtn"):SetCheck(showTotalPercentage)
	self.wndSettings:FindChild("ShowAlternativeFontBtn"):SetCheck(alternativeFont)
	self.wndSettings:FindChild("ShowClassBtn"):SetCheck(showClass)
	self.wndSettings:FindChild("AnimatedModelBtn"):SetCheck(animatedModel)
	self.wndSettings:FindChild("FadeOutCombatBtn"):SetCheck(fadeOutCombat)
end

function SlimUnitFrame:OnConfigure()
		Print("Slim UnitFrame Settings")
		self.wndSettings:Invoke()
end

-----------------------------------------------------------------------------------------------
-- SlimUnitFrame Instance
-----------------------------------------------------------------------------------------------
local SlimUnitFrameInst = SlimUnitFrame:new()
SlimUnitFrameInst:Init()
